function [alfa,R]=blfwastrain(X,Y,P)
%	BLFW+AS TRAINING
%	Trains a BLFW+AS transform from a set of paired cepstral
%	vectors, optionally considering a soft classification
%
%	[alpha,R]=blfwastrain(X,Y,P)
%
%	alpha: frequency warping factor(s) - one factor per class
%	R: amplitude scaling vector(s) - one vector per class
%
%	X: matrix of source cepstral column-vectors (without c0)
%	Y: matrix of target cepstral column-vectors - same size as X,
%      as X(:,t) is paired with Y(:,t)
%	P: soft classification matrix (optional) - P(k,t) is the
%	   probability that X(:,t) belongs to class k
%
%	---------------------------------------------------
%	Copyright (c) 2013 University of the Basque Country
%	---------------------------------------------------
%	License
%	This program is free software: you can redistribute
%	it and/or modify it under the terms of the GNU General
%	Public License as published by the Free Software Foundation,
%	either version 3 of the License, or any later version.
%	This program is distributed in the hope that it will be
%	useful, but WITHOUT ANY WARRANTY; without even the implied
%	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
%	PURPOSE. See the GNU General Public License for more details.
%	---------------------------------------------------
%	Author: Daniel Erro (derro(a)aholab.ehu.es)
%	---------------------------------------------------


[p,N]=size(X);
% if there is no P, we assume there is only one class
% otherwise, multiple classes
if nargin<3 || isempty(P)
	% calculate initial value of alpha
	R=reshape(Y-X,[],1);
	D=X.*repmat((1:p).',1,N);
	D=reshape([D(2:p,:);zeros(1,N)]-[zeros(1,N);D(1:p-1,:)],[],1);
	alfa=sum(D.*R)/sum(D.*D);
	%fprintf('alfa=%.6f ( )\n',alfa);
	% iterations until convergence is reached
	iter=100; alfanew=inf;
	while iter>0 && abs(alfanew)>0.0001
		D=blfwasconv(X,alfa);
		R=reshape(Y-D,[],1);
		D=D.*repmat((1:p).',1,N);
		D=reshape([D(2:p,:);zeros(1,N)]-[zeros(1,N);D(1:p-1,:)],[],1);
		alfanew=sum(D.*R)/sum(D.*D);
		alfa=(alfa+alfanew)/(1.0+alfa*alfanew);
		%fprintf('alfa=%.6f (dalfa=%.6f)\n',alfa,alfanew);
		iter=iter-1;
	end;
	D=[]; R=[];
	% calculate amplitude scaling vector if requested by the user
	if nargout>1 R=mean(Y-blfwasconv(X,alfa),2); end;
else
	% initial values of alpha
	m=size(P,1);
	R=reshape(Y-X,[],1);
	D=X.*repmat((1:p).',1,N);
	D=reshape([D(2:p,:);zeros(1,N)]-[zeros(1,N);D(1:p-1,:)],[],1);
	alfa=(pinv(repmat(D,1,m).*kron(P.',ones(p,1)))*R).';
	% iterations until convergence is reached
	iter=1000; alfanew=inf;
	while iter>0 && max(abs(alfanew))>0.001
		D=blfwasconv(X,alfa,[],P);
		R=reshape(Y-D,[],1);
		D=D.*repmat((1:p).',1,N);
		D=reshape([D(2:p,:);zeros(1,N)]-[zeros(1,N);D(1:p-1,:)],[],1);
		alfanew=(pinv(repmat(D,1,m).*kron(P.',ones(p,1)))*R).';
		alfa=(alfa+alfanew)./(1.0+alfa.*alfanew);
		iter=iter-1;
	end;
	D=[]; R=[];
	% calculate amplitude scaling vectors if requested by the user
	if nargout>1 R=(pinv(P.')*(Y-blfwasconv(X,alfa,[],P)).').'; end;
end;

