function [X,Y,ind1,ind2]=dtwvtln(X,Y,addD)
%	DTW WITH VTLN
%	Aligns source and target cepstral vectors via
%	dynamic time warping using vocal tract length
%	normalization for a better alignment
%
%   [Xalign,Yalign,indx,indy]=dtwvtln(X,Y,addD)
%
%	Xalign,Yalign: aligned source-target vectors
%	indx,indy: indices of the aligned vectors,
%	           Xalign=X(:,indx), Yalign=Y(:,indy)
%
%	X: source cepstral column-vectors (without c0)
%	Y: target cepstral column-vectors (without c0)
%	addD: if 1, append 1st derivatives before DTW
%
%	---------------------------------------------------
%	Copyright (c) 2013 University of the Basque Country
%	---------------------------------------------------
%	License
%	This program is free software: you can redistribute
%	it and/or modify it under the terms of the GNU General
%	Public License as published by the Free Software Foundation,
%	either version 3 of the License, or any later version.
%	This program is distributed in the hope that it will be
%	useful, but WITHOUT ANY WARRANTY; without even the implied
%	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
%	PURPOSE. See the GNU General Public License for more details.
%	---------------------------------------------------
%	Author: Daniel Erro (derro(a)aholab.ehu.es)
%	---------------------------------------------------


% initialize previous global distance
avdprev=inf;
% first call to dtw
[ind1,ind2,avd]=dtw_(X,Y,addD);
% iterations
%fprintf('DTW-START\navd=%.6f ( )\n',avd);
while avd<avdprev
	% keep copy of previous distance and indices
	avdprev=avd; ind1prev=ind1; ind2prev=ind2;
	% get vtln factor using current alignment
	alfa=blfwastrain(X(:,ind1),Y(:,ind2));
	% aplly vtln to source to make it closer to target
	Xw=blfwasconv(X,alfa);
	% dtw between warped-source and target
	[ind1,ind2,avd]=dtw_(Xw,Y,addD);
	%fprintf('avd=%.6f (alfa=%.6f)\n',avd,alfa);
end;
%fprintf('DTW-END\n');
% take previous indices (avd increased in last iteration)
ind1=ind1prev; ind2=ind2prev;
% free space
ind1prev=[]; ind2prev=[];
% select vectors from original sets
X=X(:,ind1); Y=Y(:,ind2);
% free space
if nargout<3 ind1=[]; ind2=[]; end;

% dtw_
function [ind1,ind2,avd]=dtw_(X,Y,addD)
% generic DTW alignment
N1=size(X,2);
N2=size(Y,2);
% append deltas if requested
if nargin>=3 && addD==1
	X=[X; 0.5*(X(:,[2:N1 N1])-X(:,[1 1:N1-1]))];
	Y=[Y; 0.5*(Y(:,[2:N2 N2])-Y(:,[1 1:N2-1]))];
end;
% cepstral distance matrix
d=zeros(N1,N2);
for n1=1:N1
	for n2=1:N2
		d(n1,n2)=sqrt(sum((X(:,n1)-Y(:,n2)).^2));
	end;
end;
% auxiliary matrix for backtracking
m=zeros(N1,N2,'int8');
% viterbi
for n1=1:N1
	for n2=1:N2
		if n1==1 && n2==1
			% no possible movements
			m(n1,n2)=0;
		elseif n1==1
			% only reachable from a lower n2
			m(n1,n2)=01;
			d(n1,n2)=d(n1,n2-1)+d(n1,n2);
		elseif n2==1
			% only reachable from a lower n1
			m(n1,n2)=10;
			d(n1,n2)=d(n1-1,n2)+d(n1,n2);
		else
			% reachable from 3 different points: vertical, horizontal, diagonal
			dbackup=d(n1,n2);
			d(n1,n2)=inf;
			dtry=d(n1,n2-1)+dbackup;
			if dtry<d(n1,n2) m(n1,n2)=01; d(n1,n2)=dtry; end;
			dtry=d(n1-1,n2)+dbackup;
			if dtry<d(n1,n2) m(n1,n2)=10; d(n1,n2)=dtry; end;
			dtry=d(n1-1,n2-1)+1.5*dbackup;
			if dtry<d(n1,n2) m(n1,n2)=11; d(n1,n2)=dtry; end;
		end;
	end;
end;
% global distance
avd=d(N1,N2);
% free space
d=[];
% backtracking
ind1=[]; ind2=[];
n1=N1; n2=N2;
while n1>1 || n2>1
	ind1=[n1 ind1]; ind2=[n2 ind2];
	if m(n1,n2)==01
		n2=n2-1;
	elseif m(n1,n2)==10
		n1=n1-1;
	elseif m(n1,n2)==11
		n1=n1-1; n2=n2-1;
	end;
end;
ind1=[1 ind1]; ind2=[1 ind2];
