function [G,P]=gmmtrain(X,m)
%	GMM TRAINING
%	Trains a GMM (full covariance matrices) from a
%	set of vectors
%
%	[G,P]=gmmtrain(X,m)
%
%	G: gaussian mixture model
%	P: matrix where P(k,t) is the probability
%	   that X(:,t) belongs to class k
%
%	X: matrix of column-vectors to be classified
%	m: number of classes (>1)
%
%	---------------------------------------------------
%	Copyright (c) 2013 University of the Basque Country
%	---------------------------------------------------
%	License
%	This program is free software: you can redistribute
%	it and/or modify it under the terms of the GNU General
%	Public License as published by the Free Software Foundation,
%	either version 3 of the License, or any later version.
%	This program is distributed in the hope that it will be
%	useful, but WITHOUT ANY WARRANTY; without even the implied
%	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
%	PURPOSE. See the GNU General Public License for more details.
%	---------------------------------------------------
%	Author: Daniel Erro (derro(a)aholab.ehu.es)
%	---------------------------------------------------


% calculate perturbation to avoid ill-conditioning
pert=0.02*min(var(X,1,2));
% initialize gmm
[G,P,L]=gmminit(X,m,pert);
% e-m algorithm
%fprintf('EM-START\n%.6f\n',L);
Lant=-inf; iter=100;
while L-Lant>0.000001 && iter>0
    iter=iter-1; Lant=L;
	[G,P,L]=gmmreestimate(X,G,P,pert);
	%fprintf('%.6f\n',L);
end;
%fprintf('EM-END\n',L);

% gmminit
function [G,P,L]=gmminit(X,m,pert)
% kmeans-based initialization, initial global likelihood and posteriors
N=size(X,2);
c=clustkmeans(X,m);
P=zeros(m,N);
for g=1:m P(g,find(c==g))=1.0; end;
clear c;
G(1:m)=struct('a',0.0,'u',[],'E',[],'dE',0.0,'iE',[]);
[G,P,L]=gmmreestimate(X,G,P,pert);

% gmmreestimate
function [G,P,L]=gmmreestimate(X,G,P,pert)
% reestimation of gmm parameters
[dim,N]=size(X);
for g=1:length(G)
	wPg=(1/N)*P(g,:);
	G(g).a=sum(wPg);
	wPg=repmat(wPg,dim,1);
	fact=1/G(g).a;
	G(g).u=fact*sum(wPg.*X,2);
	Xu=X-repmat(G(g).u,1,N);
	G(g).E=fact*((wPg.*Xu)*Xu.')+pert*eye(dim);
	G(g).dE=det(G(g).E);
	G(g).iE=inv(G(g).E);
end;
% update posteriors and likelihood
[P,L]=gmmprobs(G,X);

% clustkmeans
function [c,c0]=clustkmeans(X,m)
% k-means clustering
N=size(X,2);
% random initial centroids (uniformly spread)
c0=(1:m)*floor(N/m); c=zeros(1,N);
% initial classification
c=updateclassif(X,c0,c);
% iterations
iter=100;
while iter>0
    iter=iter-1;
    % update centroids given last classification
    c0=updatecentroids(X,m,c);
    % update classification given new centroids
	[c,cambios]=updateclassif(X,c0,c);
    % if nothing changed, exit
	if cambios==0 iter=0; end;
end;

% updateclassif
function [c,cambios]=updateclassif(X,c0,c)
% update classification according to input centroids
N=size(X,2);
m=length(c0);
cambios=0;
for k=1:N
	dmin=inf; kkmin=0;
	for kk=1:m
		dact=sqrt(sum((X(:,k)-X(:,c0(kk))).^2));
		if dact<dmin dmin=dact; kkmin=kk; end;
	end;
	if c(k)~=kkmin cambios=cambios+1; end;
	c(k)=kkmin;
end;

% updatecentroids
function c0=updatecentroids(X,m,c)
% update centroids according to input classification
c0=zeros(1,m);
for k=1:m
	ind=find(c==k);
	sumd=zeros(size(ind));
	for kk=1:length(ind)-1
		for kkk=kk+1:length(ind)
			dact=sqrt(sum((X(:,ind(kk))-X(:,ind(kkk))).^2));
			sumd(kk)=sumd(kk)+dact;
			sumd(kkk)=sumd(kkk)+dact;
		end;
	end;
	[dmin,kkmin]=min(sumd);
	c0(k)=ind(kkmin);
end;
